/**
 * 
 * Path representation wrapper
 */

/**
 * @class Path
 */
export default class Path {

  constructor(pathComponents) {
    this._parts = pathComponents;
  }

  get id() {
    return this._parts.length > 0 ? this._parts[this._parts.length - 1] : null;
  }

  get isDocument() {
    return this._parts.length > 0 && this._parts.length % 2 === 0;
  }

  get isCollection() {
    return this._parts.length % 2 === 1;
  }

  get relativeName() {
    return this._parts.join('/');
  }

  child(relativePath) {
    return new Path(this._parts.concat(relativePath.split('/')));
  }

  parent() {
    return this._parts.length > 0 ? new Path(this._parts.slice(0, this._parts.length - 1)) : null;
  }

  /**
   *
   * @package
   */
  static fromName(name) {
    const parts = name.split('/');
    return parts.length === 0 ? new Path([]) : new Path(parts);
  }
}